;/*
;
;   msgs.mc - example input for logged messages
;   Copyright (C) 2004  Vesely Alessandro
;
;   This program is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation; either version 2 of the License, or
;   (at your option) any later version.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.
;
;   You should have received a copy of the GNU General Public License
;   along with this program; if not, write to the Free Software
;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;*/
;
;#include <windows.h>
;/** @file
;  MC generates msgs.h from the example msgs.mc for logged messages.
;
;  The <b>Format language</b> is introduced with
;  @code
;    LanguageNames=(
;                   English=0x0409:MSG_EN
;                   Format=0x0444:MSG_FMT
;                  )
;  @endcode
;  You may insert any further language you want, but 0x444
;  is the one that log_init() loads to retrieve machine readable
;  formats. See @ref nuts for an example.
;
;  A <b>category</b> is an attribute of a reported event.
;  The categories must be numbered consecutively,
;  beginning with the number 1. Use categories to classify
;  your messages for improved readability.
;
;  Reserve the numbers until, say, 1000 for categories.
;  That gap is necessary to ensure that categories
;  form their own index entry in the binary resource
;  header. That way the number of categories, needed for
;  source registration, can be determined at runtime.
;
;  The <b>facility</b> is part of a message id: combined with
;  the severity it constitutes the high word of the message id.
;  You may use different facilities to let messages being
;  defined by different programmers. Otherwise,
;  you may initialize a log_event_data so as to automatically
;  set <i>category == facility</i> in the reported events.
;
;  Here is a simple example of a real life <code>.mc</code> file:
;  @code
;  ;#include <windows.h>
;  ;#include <w32event.h>
;  ;
;  ;// global application log
;  ;extern log_event_data the_application_log;
;  ;#define APP_LOG (&the_application_log)
;  ;#define APP_LOG_NAME "my_name"
;  ;#define APP_LOG_DEFINE log_event_data the_application_log
;  ;
;  ;// macro to report MEMORY_FAILURE
;  ;#define REPORT_MEMORY_FAILURE(REQ_SIZE) do {                 \
;  ; unsigned const the_requested_size = (REQ_SIZE);             \
;  ; ReportEvent(the_application_log.log,                        \
;  ;     EVENTLOG_ERROR_TYPE, PROCESS_MESSAGES, MEMORY_FAILURE,  \
;  ;     the_application_log.sid, 0,                             \
;  ;     sizeof the_requested_size, NULL, &the_requested_size);  \
;  ;} while (0)
;     @endcode
;
;  @see v_log_event()
;  @see log_init()
;*/

MessageIdTypedef = DWORD

SeverityNames=(
    Success=0x0
    Informational=0x1
    Warning=0x2
    Error=0x3
    )

;// keep facility and category sync for facility==category
FacilityNames=(
    NoFacility=0
    Facility_1=1
    Facility_2=2
    Facility_3=3
    )

;//LanguageNames = (name=number:filename)
LanguageNames=(
    English=0x0409:MSG_EN
    Format=0x0444:MSG_FMT
    )

MessageId    = 1
Facility     = NoFacility
SymbolicName = CATEGORY_1
Language     = English
Category 1
.
Language     = Format
empty
.

MessageId    = 2
SymbolicName = CATEGORY_2
Language     = English
Category 2
.
Language     = Format
empty
.


MessageId    = 3
SymbolicName = CATEGORY_3
Language     = English
Category 3
.
Language     = Format
empty
.


;// FIRST MESSAGE HAS VALUE 1000
;// Severity and Facility will stick to next messages
;// while MessageId is increased automatically
MessageId    = 1000
Severity     = Success
Facility     = Facility_1
SymbolicName = FIRST_MESSAGE
Language     = English
%n
This is the first message that I write%n
it spans multiple lines%n
%n
(also has white lines)%0
.
Language     = Format
empty
.

MessageId    =
SymbolicName = MESSAGE_WITH_AN_ARGUMENT_s
Severity     = Informational
Language     = English
The argument is %1. It is inserted.
.
Language     = Format
%s
.

MessageId    =
SymbolicName = MESSAGE_VARIOUS_ARGUMENTS_sMsasMD
Severity     = Warning
Language     = English
%n
This message has various arguments, as can be seen
from the tail of its symbolic name:%n
%n
              MESSAGE_VARIOUS_ARGUMENTS_sMsasMD%n
%n
where I use%n
D to mean a big integer and%n
a to mean an asterix (integer size or precision argument.)%n
%n
First argument is %1.%n
The next argument is a system error message: %2.%n
Next come two strings%n
- %3%n
- %4%n
Another system error, curiously it is: %5%n
Finally the sixth argument: a number %6.%n
%n
Note that in a different language, the fields may
need to be inserted in a different order, so the
numbering here corresponds to the order of the
arguments in the original call. In the format language
the fields have to be exactly in the same order.%n
%n
The trailing dot is for end of message text.%0
.
Language     = Format
%s%M%s%.*s%M%I64d
.

MessageId    =
SymbolicName = MESSAGE_BAD_FORMAT_s
Severity     = Error
Language     = English
%n
A percent sign is %% The argument is %1, a string.%0
.
Language     = Format
What happens if this is wrong? And with %%?
And with a spare %? %s must still be here.
.

MessageId    =
SymbolicName = MESSAGE_TOO_MANY_ARGS_ssssssssss
Facility     = Facility_2
Severity     = Warning
Language     = English
%n
In the test version, I reset MAX_INSERTIONS to 8.
(Normally it is 32, because I use flags it cannot
be more that the bits in an unsigned int.)

So this %1 %2 %3 %4 %5 %6 %7 %8 %9 %10 will not work:
the last two insertions will miss.%0
.
Language     = Format
%s%s%s%s%s
%s%s%s%s%s
.

MessageId    =
SymbolicName = MESSAGE_TYPES_diouxXfe
Language     = English
%n
decimal %1%n
integer %2%n
octal %3%n
unsigned %4%n
hex %5%n
Hex %6%n
float %7%n
exponentiated %8%0
.
Language     = Format
%5d%i%o%u%#x%#020X%+8.5f%e
.

MessageId    =
SymbolicName = MESSAGE_TYPES_EgGcCsSp
Language     = English
%n
Exponentiated %1%n
go auto e or f %2%n
Go auto E or f %3%n
character %4%n
Character (unicode)%5%n
string %6%n
String %7%n
pointer %8%0
.
Language     = Format
%E%g%G%c%C%s%S%p
.


MessageId    =
SymbolicName = MESSAGE_WRONG_COUNT_spds
Facility     = Facility_3
Severity     = Informational
Language     = English
%n
This has 4 arguments but in the english text
I refer to 5:%n
1 %1%n
2 %2%n
3 %3%n
4 %4%n
5 %5%0
.
Language     = Format
%s%p%d%s
.

MessageId    =
SymbolicName = MESSAGE_WRONG_COUNT_spd
Severity     = Warning
Language     = English
%n
This has 3 arguments but in the english text
I refer to 2:%n
1 %1%n
2 %2%0
.
Language     = Format
%s%p%d
.
