/*

    w32event.h - utility functions to log application event
    Copyright (C) 2004  Vesely Alessandro

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include <windows.h>
#include <stdarg.h>
#if !defined(W32EVENT_H_INCLUDED)
#if defined(__cplusplus)
extern "C" {
#endif

/**
* @file
* declarations for utility functions.
*/

HANDLE SetEventSource(
   LPTSTR pszSrcName, // event source name
   LPTSTR pszMsgDLL,  // name or path for message DLL
   DWORD  dwNum);     // number of categories

PSID GetEventSid(void);
void ClearEventSid(PSID);

/**
* structure of the message resource (guess)
* @see ArkadiyOlovyannikov
*/
struct log_message_table;
typedef struct log_message_table log_message_table;

/**
* Structure for log resources. That data has to be allocated
* by the user before initialization. The members are exposed
* so that they can be used independently of the functions
* defined in this module, e.g. to call <code>ReportEvent</code>
* directly.
*
* Note that log_event(), log_event_blob(), and v_log_event()
* consider this structure as read-only data. Therefore you
* may safely make a copy of the structure and use that instead
* of the original.
*
* @see log_init()
*/
typedef struct log_event_data
{
   HANDLE log;   ///< As returned from SetEventSource
   PSID sid;     ///< As returned from GetEventSid
   HMODULE msg;  ///< Handle of loaded messages. Used for cleanup.
   HGLOBAL res;  ///< Resource handle of loaded messages. Used for cleanup.

   /**
   * Pointer to MESSAGETABLE binary data for the Format language.
   * This is the pointer obtained from LockResource.
   */
   log_message_table *table;

   /**
   * Number of categories. This is read at initialization time
   * from the first MESSAGETABLE index entry.
   */
   int last_category;

   /**
   * Category passed to log_init().
   */
   int category;
} log_event_data;

/**
* Maximum number of insertion strings. This is the maximum size
* of an array of string passed to <code>ReportEvent</code>.
* @see v_log_event()
*/
#define MAX_INSERTIONS 32

int log_init(log_event_data *log, // target structure to initialize
   int category,      // category > 0, 0 for category == facility
   LPTSTR pszSrcName, // event source name
   LPTSTR pszMsgDLL); // name or path for message DLL
void log_clear(log_event_data *log);
void log_event(log_event_data const *log, DWORD msg, ... /* insertion strings */);
void log_event_blob(log_event_data const*log, DWORD msg,
   DWORD blob_length, void *blob, ... /* insertion strings */);
void v_log_event(log_event_data const*log,
   DWORD msg, DWORD b_len, void *blob, va_list args);

#if defined(__cplusplus)
}
#endif
#define W32EVENT_H_INCLUDED
#endif
